package xyz.lychee.lagfixer.libs.adventure.resource;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.util.MonkeyBars;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

final class ResourcePackRequestImpl implements ResourcePackRequest {
   private final List<ResourcePackInfo> packs;
   private final ResourcePackCallback cb;
   private final boolean replace;
   private final boolean required;
   private final @Nullable Component prompt;

   ResourcePackRequestImpl(final List<ResourcePackInfo> packs, final ResourcePackCallback cb, final boolean replace, final boolean required, final @Nullable Component prompt) {
      this.packs = packs;
      this.cb = cb;
      this.replace = replace;
      this.required = required;
      this.prompt = prompt;
   }

   public @NotNull List<ResourcePackInfo> packs() {
      return this.packs;
   }

   public @NotNull ResourcePackRequest packs(final @NotNull Iterable<? extends ResourcePackInfoLike> packs) {
      return this.packs.equals(packs) ? this : new ResourcePackRequestImpl(MonkeyBars.toUnmodifiableList(ResourcePackInfoLike::asResourcePackInfo, packs), this.cb, this.replace, this.required, this.prompt);
   }

   public @NotNull ResourcePackCallback callback() {
      return this.cb;
   }

   public @NotNull ResourcePackRequest callback(final @NotNull ResourcePackCallback cb) {
      return cb == this.cb ? this : new ResourcePackRequestImpl(this.packs, (ResourcePackCallback)Objects.requireNonNull(cb, "cb"), this.replace, this.required, this.prompt);
   }

   public boolean replace() {
      return this.replace;
   }

   public boolean required() {
      return this.required;
   }

   public @Nullable Component prompt() {
      return this.prompt;
   }

   public @NotNull ResourcePackRequest replace(final boolean replace) {
      return replace == this.replace ? this : new ResourcePackRequestImpl(this.packs, this.cb, replace, this.required, this.prompt);
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (other != null && this.getClass() == other.getClass()) {
         ResourcePackRequestImpl that = (ResourcePackRequestImpl)other;
         return this.replace == that.replace && Objects.equals(this.packs, that.packs) && Objects.equals(this.cb, that.cb) && this.required == that.required && Objects.equals(this.prompt, that.prompt);
      } else {
         return false;
      }
   }

   public int hashCode() {
      return Objects.hash(new Object[]{this.packs, this.cb, this.replace, this.required, this.prompt});
   }

   public @NotNull String toString() {
      return Internals.toString(this);
   }

   public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("packs", this.packs), ExaminableProperty.of("callback", this.cb), ExaminableProperty.of("replace", this.replace), ExaminableProperty.of("required", this.required), ExaminableProperty.of("prompt", this.prompt));
   }

   static final class BuilderImpl implements ResourcePackRequest.Builder {
      private List<ResourcePackInfo> packs;
      private ResourcePackCallback cb;
      private boolean replace;
      private boolean required;
      private @Nullable Component prompt;

      BuilderImpl() {
         this.packs = Collections.emptyList();
         this.cb = ResourcePackCallback.noOp();
         this.replace = false;
      }

      BuilderImpl(final @NotNull ResourcePackRequest req) {
         this.packs = req.packs();
         this.cb = req.callback();
         this.replace = req.replace();
         this.required = req.required();
         this.prompt = req.prompt();
      }

      @NotNull
      public ResourcePackRequest.@NotNull Builder packs(final @NotNull ResourcePackInfoLike first, final @NotNull ResourcePackInfoLike... others) {
         this.packs = MonkeyBars.nonEmptyArrayToList(ResourcePackInfoLike::asResourcePackInfo, first, others);
         return this;
      }

      @NotNull
      public ResourcePackRequest.@NotNull Builder packs(final @NotNull Iterable<? extends ResourcePackInfoLike> packs) {
         this.packs = MonkeyBars.toUnmodifiableList(ResourcePackInfoLike::asResourcePackInfo, packs);
         return this;
      }

      @NotNull
      public ResourcePackRequest.@NotNull Builder callback(final @NotNull ResourcePackCallback cb) {
         this.cb = (ResourcePackCallback)Objects.requireNonNull(cb, "cb");
         return this;
      }

      @NotNull
      public ResourcePackRequest.@NotNull Builder replace(final boolean replace) {
         this.replace = replace;
         return this;
      }

      @NotNull
      public ResourcePackRequest.@NotNull Builder required(final boolean required) {
         this.required = required;
         return this;
      }

      @NotNull
      public ResourcePackRequest.@NotNull Builder prompt(final @Nullable Component prompt) {
         this.prompt = prompt;
         return this;
      }

      public @NotNull ResourcePackRequest build() {
         return new ResourcePackRequestImpl(this.packs, this.cb, this.replace, this.required, this.prompt);
      }
   }
}
